/**
 * $Id: AboutCanvas.java 33 2006-03-22 18:22:13Z Gerhard $
 * 
 * Copyright (C) 2006 Gerhard Zlabinger (gerhard.zlabinger@gmail.com)
 *
 * This file is part of Ubertris.
 * 
 * Ubertris is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Ubertris is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Ubertris; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 */

package ubertris;

import javax.microedition.lcdui.Canvas;
import javax.microedition.lcdui.Command;
import javax.microedition.lcdui.CommandListener;
import javax.microedition.lcdui.Display;
import javax.microedition.lcdui.Displayable;
import javax.microedition.lcdui.Font;
import javax.microedition.lcdui.Graphics;

final class AboutCanvas extends Canvas implements CommandListener {

    private final UbertrisCanvas canvas;

    public AboutCanvas(UbertrisCanvas canvas) {
        this.canvas = canvas;
    }

    protected void showNotify() {
        addCommand(new Command("Back", Command.BACK, 1));
        setCommandListener(this);
    }

    public void commandAction(Command cmd, Displayable disp) {
        Display.getDisplay(canvas.midlet).setCurrent(canvas);
    }

    private void paintAppProperty(String property, Graphics g, int x, int y) {
        final String value = this.canvas.midlet.getAppProperty(property);

        if (value != null)
            g.drawString(value, x, y, Graphics.HCENTER | Graphics.TOP);
    }
    
    private void addSystemProperty(String property, Graphics g, int y) {
        final String value = System.getProperty("microedition." + property);

        if (value != null && value.length() > 0) {
            g.drawString(property + ": " + value, 0, y, Graphics.LEFT | Graphics.TOP);
        }
    }

    protected void paint(Graphics g) {
        final int width = this.canvas.getWidth();

        g.setColor(UbertrisCanvas.COLOR_BG);
        g.fillRect(0, 0, width, this.canvas.getHeight());

        final Font font = Font.getFont(Font.FACE_PROPORTIONAL, Font.STYLE_PLAIN, Font.SIZE_SMALL);
        final int fontHeight = font.getHeight();

        g.setColor(UbertrisCanvas.COLOR_TEXT);
        g.setFont(font);
        paintAppProperty("MIDlet-Name", g, width >> 1, 0);
        paintAppProperty("MIDlet-Version", g, width >> 1, fontHeight);
        paintAppProperty("MIDlet-Vendor", g, width >> 1, fontHeight << 1);
        paintAppProperty("MIDlet-Info-URL", g, width >> 1, fontHeight * 4);

        g.drawLine(0, fontHeight * 6, width, fontHeight * 6);

        // system stuff
        
        int y = fontHeight * 6;

        final Runtime runtime = Runtime.getRuntime();
        g.drawString("Total: " + runtime.totalMemory(), 0, y += fontHeight, Graphics.LEFT | Graphics.TOP);
        g.drawString("Free: " + runtime.freeMemory(), 0, y += fontHeight, Graphics.LEFT | Graphics.TOP);
        
        addSystemProperty("configuration", g, y += fontHeight);
        addSystemProperty("profiles", g, y += fontHeight);
        addSystemProperty("platform", g, y += fontHeight);
        addSystemProperty("locale", g, y += fontHeight);
        addSystemProperty("jtwi.version", g, y += fontHeight);
    }
}
