/**
 * $Id: GameStore.java 36 2006-03-22 19:10:08Z Gerhard $
 * 
 * Copyright (C) 2006 Gerhard Zlabinger (gerhard.zlabinger@gmail.com)
 *
 * This file is part of Ubertris.
 * 
 * Ubertris is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Ubertris is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Ubertris; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 */

package ubertris;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;

import javax.microedition.rms.RecordStoreException;

final class GameStore extends SingleRecordStore {

  protected String getStoreName() {
    return "savedGame";
  }

  public boolean isGameAvailable() {    
    return isRecordAvailable();
  }

  public void storeGame(GameThread gameThread, Field field) {
    final ByteArrayOutputStream bos = new ByteArrayOutputStream();
    final DataOutputStream      dos = new DataOutputStream(bos);

    try {
      gameThread.serialize(dos);
      field.serialize(dos);
      setRecord(bos.toByteArray());
    } catch (RecordStoreException e) {
      e.printStackTrace();
    } catch (IOException e) {
      e.printStackTrace();
    } finally  {
      try {
        dos.close();
      } catch (IOException e) {
        e.printStackTrace();
      }
    
      try {
        bos.close();
      } catch (IOException e) {
        e.printStackTrace();
      }
    }
  }

  /*
   * TODO: remove workaround class and find elegant solution. (E.g. by encapsulating the GameState properly...)
   */
  static final class GameState {
    public final Field      field;
    public final GameThread gameThread;

    public GameState(GameThread gameThread, Field field) {
      this.gameThread = gameThread;
      this.field = field;
    }
  }

  public GameState restoreGame(UbertrisCanvas canvas) throws IOException, RecordStoreException {
    ByteArrayInputStream bis = null;
    DataInputStream      dis = null;

    try {
      bis = new ByteArrayInputStream(getRecord());
      dis = new DataInputStream(bis);
      return new GameState(GameThread.deserialize(canvas, dis), new Field(canvas, dis));    
    } finally {
      if (dis != null) {
        try {
          dis.close();
        } catch (IOException ex) {
          ex.printStackTrace();
        }
      }

      if (bis != null) {
        try {
          bis.close();
        } catch (IOException ex) {
          ex.printStackTrace();
        }
      }
    }
  }
}
