/**
 * $Id: HighscoreManager.java 39 2006-08-02 08:42:15Z gez $
 * 
 * Copyright (C) 2006 Gerhard Zlabinger (gerhard.zlabinger@gmail.com)
 *
 * This file is part of Ubertris.
 * 
 * Ubertris is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Ubertris is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Ubertris; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 */

package ubertris;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;

import javax.microedition.rms.RecordStoreException;

/**
 * @author Gerhard
 * 
 */
final class HighscoreManager extends SingleRecordStore {

    protected String getStoreName() {
      return "highScore";
    }

    public int loadHighScore() {
      ByteArrayInputStream bis = null;
      DataInputStream dis = null;

      try {
        final byte[] data = getRecord();
      
        if (data == null)
            return 0;
      
        bis = new ByteArrayInputStream(data);
        dis = new DataInputStream(bis);
        return dis.readInt();
      } catch (RecordStoreException ex) {
        ex.printStackTrace();
        return 0;
      } catch (IOException ex) {
        ex.printStackTrace();
        return 0;
      } finally {
        if (dis != null) {
          try {
            dis.close();
          } catch (IOException e1) {
            e1.printStackTrace();
          }
        }

        if (bis != null) {
          try {
            bis.close();
          } catch (IOException e1) {
            e1.printStackTrace();
          }
        }
      }
    }

    public void saveHighScore(int highScore) {
      ByteArrayOutputStream bos = null;
      DataOutputStream dos = null;

      try {
        bos = new ByteArrayOutputStream();
        dos = new DataOutputStream(bos);
        dos.writeInt(highScore);
        setRecord(bos.toByteArray());
      } catch (RecordStoreException ex) {
        ex.printStackTrace();
      } catch (IOException ex) {
        ex.printStackTrace();
      } finally {
        if (dos != null) {
          try {
            dos.close();
          } catch (IOException e2) {
            e2.printStackTrace();
          }
        }

        if (bos != null) {
          try {
            bos.close();
          } catch (IOException e1) {
            e1.printStackTrace();
          }
        }
      }
    }
}
