/**
 * $Id: SingleRecordStore.java 33 2006-03-22 18:22:13Z Gerhard $
 * 
 * Copyright (C) 2006 Gerhard Zlabinger (gerhard.zlabinger@gmail.com)
 *
 * This file is part of Ubertris.
 * 
 * Ubertris is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Ubertris is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Ubertris; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 */

package ubertris;

import javax.microedition.rms.RecordEnumeration;
import javax.microedition.rms.RecordStore;
import javax.microedition.rms.RecordStoreException;
import javax.microedition.rms.RecordStoreFullException;
import javax.microedition.rms.RecordStoreNotFoundException;

abstract class SingleRecordStore {

  protected abstract String getStoreName();

  protected RecordStore openStore()
    throws  RecordStoreFullException,
            RecordStoreNotFoundException,
            RecordStoreException {
    return RecordStore.openRecordStore(getStoreName(), true);
  }

  protected final boolean isRecordAvailable() {
    RecordStore store = null;
    try {
      store = openStore();
      
      if (store.getNumRecords() > 0)
        return true;
    } catch (RecordStoreException e) {
      e.printStackTrace();
    } finally {
      if (store != null) {
        try {
          store.closeRecordStore();
        } catch (RecordStoreException e) {
          e.printStackTrace();
        }
      }
    }

    return false;
  }

  public final void nukeStore() {
    try {
      RecordStore.deleteRecordStore(getStoreName());
    } catch (RecordStoreException e) {
      e.printStackTrace();
    }
  }

  protected final void setRecord(byte[] data) throws RecordStoreException {
    RecordStore store = null;
    RecordEnumeration e = null;

    try {
      store = openStore();
      
      if (store.getNumRecords() == 0) {
        store.addRecord(data, 0, data.length);
      } else {
        e = store.enumerateRecords(null, null, false);
        store.setRecord(e.nextRecordId(), data, 0, data.length);
      }
    } finally {
      if (e != null)
       e.destroy();

      if (store != null) {
        try {
          store.closeRecordStore();
        } catch (RecordStoreException ex) {
          ex.printStackTrace();
        }
      }
    }
  }

  protected final byte[] getRecord() throws RecordStoreException {
    RecordStore store = null;
    RecordEnumeration e = null;

    try {
      store = openStore();

      if (store.getNumRecords() == 0)
        return null;

      e = store.enumerateRecords(null, null, false);
      return store.getRecord(e.nextRecordId());
    } finally {
      if (e != null)
        e.destroy();

      if (store != null) {
        try {
          store.closeRecordStore();
        } catch (RecordStoreException ex) {
          ex.printStackTrace();
        }
      }
    }
  }
}
