/**
 * $Id: UbertrisCanvas.java 45 2006-08-16 17:50:39Z gez $
 * 
 * Copyright (C) 2006 Gerhard Zlabinger (gerhard.zlabinger@gmail.com)
 *
 * This file is part of Ubertris.
 * 
 * Ubertris is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Ubertris is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Ubertris; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 */

package ubertris;

import java.io.IOException;

import javax.microedition.lcdui.Alert;
import javax.microedition.lcdui.AlertType;
import javax.microedition.lcdui.Command;
import javax.microedition.lcdui.CommandListener;
import javax.microedition.lcdui.Display;
import javax.microedition.lcdui.Displayable;
import javax.microedition.lcdui.Font;
import javax.microedition.lcdui.Graphics;
import javax.microedition.lcdui.game.GameCanvas;
import javax.microedition.midlet.MIDletStateChangeException;
import javax.microedition.rms.RecordStoreException;

final class UbertrisCanvas extends GameCanvas implements CommandListener {

    /*
     * synchronization: this.notify() --> status should be changed or has been
     * changed keyQueue.notify() --> new keyevent
     */

    public static final int  COLOR_BG           = 0x000000;
    public static final int  COLOR_FIELD_BG     = 0x00003F;
    public static final int  COLOR_SCORE        = 0xFFBB00;
    public static final int  COLOR_TEXT         = 0xFFFFFF;

    private static final int PROGRESS_BAR_WIDTH = 4;

    int                      width, height;
    final Graphics           buffer             = getGraphics();
    final Field              field;
    final UbertrisMIDlet     midlet;

    private final Font       gameFont           = Font.getFont(Font.FACE_PROPORTIONAL, Font.STYLE_PLAIN, Font.SIZE_SMALL);
    private final int        gameFontHeight     = gameFont.getHeight();
    private int              textCenter;
    private int              textLeft;
    private int              textWidth;

    private GameThread       gameThread;

    private final Command    cmdResume          = new Command("Resume", "Resume Game", Command.OK, 1);
    private final Command    cmdPause           = new Command("Pause", "Pause Game", Command.STOP, 2);
    private final Command    cmdNewGame         = new Command("New", "New Game", Command.CANCEL, 3);
    private final Command    cmdAbout           = new Command("About", "About Ubertris", Command.SCREEN, 4);
    private final Command    cmdExit            = new Command("Exit", "Exit Ubertris", Command.EXIT, 5);
    private final Command    cmdNukeHighscore   = new Command("Nuke Highscore", Command.SCREEN, 6);

    private int              highScore;
    private int              previousHighScore;
    
    // FUCK MOTOROLA: V360 crashes if flushGraphics() is called while canvas is
    // not being shown
    private boolean          isHidden           = true;

    // TODO: this should be replaced by some kind of state tracking mechanism
    private boolean          gameOver;

    public UbertrisCanvas(UbertrisMIDlet midlet) {
        super(false);

        this.midlet = midlet;

        setFullScreenMode(true);

        addCommand(cmdExit);
        addCommand(cmdAbout);
        addCommand(cmdNukeHighscore);
        addCommand(cmdPause);
        addCommand(cmdNewGame);
        setCommandListener(this);

        /*
         * restore previous game
         */
        final GameStore gameStore = new GameStore();
        if (gameStore.isGameAvailable()) {
            GameStore.GameState state = null;
            try {
              state = gameStore.restoreGame(this);
            } catch (RecordStoreException e) {
              e.printStackTrace();
            } catch (IOException e) {
              e.printStackTrace();
            }

            if (state != null) {
              field = state.field;
              gameThread = state.gameThread;

              removeCommand(cmdPause);
              addCommand(cmdResume);
            } else {
              field = new Field(this);
              gameThread = new GameThread(this);
            }

            gameStore.nukeStore();
        } else {
          field = new Field(this);
          gameThread = new GameThread(this);
        }

        // TODO: fix this cruelty
        previousHighScore = highScore = new HighscoreManager().loadHighScore();

        /*
         * we cannot rely on sizeChanged() to be called at least once, although otherwise stated in the J2ME-Specification (even the WTK emulator fails here)
         */
        sizeChanged(getWidth(), getHeight());

        /*
         * let's roll
         */
        gameThread.start();
    }

    protected void showNotify() {
        // FUCK MOTOROLA
        isHidden = false;
        flushGraphics();
    }

    protected void hideNotify() {
        // FUCK MOTOROLA
        isHidden = true;
        gameThread.pause();
    }
    
    protected void sizeChanged(int width, int height) {

        if (field == null || (this.width == width && this.height == height))
            return;

        this.width = width;
        this.height = height;

        field.calculateSizes();

        textWidth = width - field.SCREEN_WIDTH - PROGRESS_BAR_WIDTH;
        textLeft = field.SCREEN_WIDTH + PROGRESS_BAR_WIDTH;
        textCenter = textLeft + textWidth / 2;

        completelyRedrawScreen();
        flushGraphics();
    }

    // FUCK MOTOROLA
    public void flushGraphics() {
        if (!isHidden)
            super.flushGraphics();
    }

    // FUCK MOTOROLA
    public void flushGraphics(int x, int y, int width, int height) {
        if (!isHidden)
            super.flushGraphics(x, y, width, height);
    }

    private void enqueueKey(int keyCode) {
        gameThread.enqueueGameAction(getGameAction(keyCode));
    }

    protected void keyPressed(int keyCode) {
        enqueueKey(keyCode);
    }

    protected void keyRepeated(int keyCode) {
        enqueueKey(keyCode);
    }

    private void drawPopup(String text) {
        buffer.setColor(0xff0000);
        buffer.fillRoundRect(5, ((height - gameFontHeight) >> 1) - 5, width - 9, gameFontHeight + 11, 15, 15);

        buffer.setColor(COLOR_TEXT);
        buffer.drawRoundRect(5, ((height - gameFontHeight) >> 1) - 5, width - 10, gameFontHeight + 10, 15, 15);
        buffer.drawString(text, width >> 1, (height >> 1) - gameFontHeight / 2 + 1, Graphics.HCENTER | Graphics.TOP);
    }

    public void commandAction(Command cmd, Displayable disp) {
        if (cmd == cmdExit) {
            try {
                midlet.destroyApp(false);
            } catch (MIDletStateChangeException e) {
                System.err.println(e);
            }
        } else if (cmd == cmdResume) {

            removeCommand(cmdResume);
            addCommand(cmdPause);

            gameThread.resume();

        } else if (cmd == cmdPause) {
            gameThread.pause();

            removeCommand(cmdPause);
            addCommand(cmdResume);

            // now is a good time to colllect garbage, i'd say...
            System.gc();
        } else if (cmd == cmdNewGame) {

            removeCommand(cmdResume);
            addCommand(cmdPause);

            if (gameThread != null) {
                gameThread.kill();
                try {
                    gameThread.join();
                } catch (InterruptedException e) {
                    e.printStackTrace();
                }
            }

            field.reset();

            gameOver = false;
            gameThread = new GameThread(this);
            gameThread.start();

            completelyRedrawScreen();
            flushGraphics();
        } else if (cmd == cmdAbout) {
            pauseGame();

            Display.getDisplay(midlet).setCurrent(new AboutCanvas(this));

            // now is a good time to colllect garbage, i'd say...
            System.gc();
        } else if (cmd == cmdNukeHighscore) {
            pauseGame();

            final Alert alert = new Alert("Confirm Nuke", "Are You Sure?", null, AlertType.CONFIRMATION);

            final Command cmdYes = new Command("Yes", Command.OK, 1);

            alert.addCommand(cmdYes);
            alert.addCommand(new Command("No", Command.CANCEL, 2));

            alert.setCommandListener(new CommandListener() {

                public void commandAction(Command cmd, Displayable arg) {
                    if (cmd == cmdYes) {
                        // TODO: fix this cruelty
                        new HighscoreManager().nukeStore();

                        highScore = previousHighScore = 0;
                        renderHighscore();
                    }

                    Display.getDisplay(midlet).setCurrent(UbertrisCanvas.this);
                }

            });

            Display.getDisplay(midlet).setCurrent(alert);
        }
    }

    public void exit() {
        gameThread.kill();
       
        try {
          gameThread.join();
        } catch (InterruptedException e) {
          e.printStackTrace();
        }

        if (!gameOver)
          // TODO: fix cruelty
          new GameStore().storeGame(gameThread, field);
    }

    public void createLayout() {
        // FIXME: this gets called twice on startup...

        buffer.setColor(COLOR_BG);
        buffer.fillRect(0, 0, width, height);

        buffer.setColor(COLOR_TEXT);
        buffer.setFont(gameFont);
        buffer.drawString("Level", textCenter, 0, Graphics.TOP | Graphics.HCENTER);
        buffer.drawString("Score", textCenter, gameFontHeight << 1, Graphics.TOP | Graphics.HCENTER);
        buffer.drawString("Highscore", textCenter, gameFontHeight << 2, Graphics.TOP | Graphics.HCENTER);
        buffer.drawString("Next", textCenter, field.PREVIEW_SCREEN_TOP - 2, Graphics.BOTTOM | Graphics.HCENTER);
    }

    protected void completelyRedrawScreen() {
        createLayout();

        field.render();
        field.renderTilePreview(gameThread.tileNext);

        renderScore(gameThread.score);
        renderLevel(gameThread.level);
        renderHighscore();
        renderBar();
    }

    private void drawScore(int line, int score) {
        buffer.setColor(COLOR_BG);
        buffer.fillRect(textLeft, gameFontHeight * line, textWidth, gameFontHeight);

        buffer.setColor(COLOR_SCORE);
        buffer.drawString(String.valueOf(score), textCenter, gameFontHeight * line, Graphics.TOP | Graphics.HCENTER);
    }

    public void renderLevel(int level) {
        drawScore(1, level + 1);
    }

    public void renderLevel() {
        renderLevel(gameThread.level);
    }

    public boolean renderScore(int score) {
        drawScore(3, score);

        if (score >= highScore) {
            highScore = score;
            renderHighscore();
            return true;
        }

        return false;
    }

    public boolean renderScore() {
        return renderScore(gameThread.score);
    }

    private void renderHighscore() {
        drawScore(5, highScore);
    }

    public void tileLanded(Tile sender, int pointsAwarded) {
        gameThread.tileLanded(sender, pointsAwarded);
    }

    /*
     * game state callbacks
     */

    public void gameOver() {
        gameThread.gameOver();
        gameOver = true;

        drawPopup("GAME OVER");

        flushGraphics();

        if (highScore > previousHighScore) {
            // TODO: fix this cruelty
            new HighscoreManager().saveHighScore(highScore);

            // TODO: should this really be done here?
            previousHighScore = highScore;
        }

        removeCommand(cmdPause);
        removeCommand(cmdResume);

        // now is a good time to colllect garbage, i'd say...
        System.gc();
    }

    public void paused() {
        drawPopup("PAUSED");
        flushGraphics();
    }

    public void pauseGame() {
        gameThread.pause();

        // TODO: don't do this if we're gameover
        if (!gameOver) {
            removeCommand(cmdPause);
            addCommand(cmdResume);
        }
    }

    public void renderBar() {
        renderBar(gameThread.tileCount % GameThread.TILES_PER_LEVEL);
    }

    public void renderBar(int tilesInLevel) {
        final int barHeight = field.SCREEN_HEIGHT * tilesInLevel / (GameThread.TILES_PER_LEVEL - 1);
        
        /* don't forget to change PROGRESS_BAR_WIDTH if you change anything in here */
        buffer.setColor(0x00BF00);
        buffer.drawLine(field.SCREEN_WIDTH, height - barHeight, field.SCREEN_WIDTH, height);

        buffer.setColor(0x00DF00);
        buffer.drawLine(field.SCREEN_WIDTH + 1, height - barHeight, field.SCREEN_WIDTH + 1, height);

        buffer.setColor(0x00FF00);
        buffer.drawLine(field.SCREEN_WIDTH + 2, height - barHeight, field.SCREEN_WIDTH + 2, height);

        buffer.setColor(0x00DF00);
        buffer.drawLine(field.SCREEN_WIDTH + 3, height - barHeight, field.SCREEN_WIDTH + 3, height);
    }

    public void clearBar() {
        buffer.setColor(UbertrisCanvas.COLOR_BG);
        buffer.fillRect(field.SCREEN_WIDTH, field.SCREEN_TOP, PROGRESS_BAR_WIDTH, field.SCREEN_HEIGHT);
    }
}
